/*-
 * Copyright (C)2003..2011 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2003..2011 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: open.c,v 1.17 2013/01/26 11:17:03 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <babolo/BLINflag.h>
#include "mife_int.h"
#include "mife.h"

#define TRUFLAG(A) ( (A)                                                                              \
                   & ( MIFE_STRI                                                                      \
                     | MIFE_RTRY                                                                      \
                     | MIFE_BUFL                                                                      \
                     | MIFE_SLID                                                                      \
                     | MIFE_EOFL                                                                      \
                     | MIFE_FULL                                                                      \
                     | MIFE_BUFX                                                                      \
                     | BLIN_VERMASK                                                                   \
                   ) )

BLIN_flag mife_default = BLIN_VER1;

static ssize_t
mindelta(BLIN_flag flags) {
    ssize_t s;
    s = 0;
    if  (getMIFE_BUFL(flags)) s = 1 << (getMIFE_BUFL(flags) + 16);
    if  (s < MINDELTA) s = MINDELTA;
    return(s);
}

mife_descriptor *
mife_open(BLIN_flag flags, const char *path) {
    mife_descriptor *mife = NULL;
    ifBLIN_FLAG = flags;
    int er;

    ifBLIN_QX2("+ flags=%08X, path=%s\n", flags, path);
    SETBDEF(flags);
    if  (!(mife = malloc(sizeof(mife_descriptor)))) {
        errno = ENOMEM;
        ifBLIN_QW0("#1");
        goto out;
    }
    bzero(mife, sizeof(mife_descriptor));
    mife->flag = TRUFLAG(flags) | MIFE_CLOS | MIFE_PIPE;
    mife->mindelta = mindelta(mife->flag);
    if  ((mife->fd = open(path, O_RDONLY)) < 0) {
        er = errno;
        ifBLIN_QW0("#2");
        free(mife);
        mife = NULL;
        errno = er;
        goto out;
    }
    if  (!getMIFE_RTRY(mife->flag)) setMIFE_RTRY(mife->flag, NRETRY00);
    ifBLIN_QX2( "- flags=%08X, mindelta=%08X, fd=%d\n"
              , mife->flag, (u_int32_t)(mife->mindelta), mife->fd
              )
    ;
out:
    return(mife);
}

mife_descriptor *
mife_opef(BLIN_flag flags, int fd) {
    mife_descriptor *mife = NULL;
    ifBLIN_FLAG = flags;

    ifBLIN_QX2("+ flags=%08X, fd=%d\n", flags, fd);
    SETBDEF(flags);
    if  (!(mife = malloc(sizeof(mife_descriptor)))) {
        errno = ENOMEM;
        ifBLIN_QW0("#1");
        goto out;
    }
    bzero(mife, sizeof(mife_descriptor));
    mife->flag = TRUFLAG(flags) | MIFE_PIPE;
    mife->mindelta = mindelta(mife->flag);
    mife->fd = fd;
    if  (!getMIFE_RTRY(mife->flag)) setMIFE_RTRY(mife->flag, NRETRY00);
    ifBLIN_QX2( "- flags=%08X, mindelta=%08X, fd=%d\n"
              , mife->flag, (u_int32_t)(mife->mindelta), mife->fd
              )
    ;
out:
    return(mife);
}

mife_descriptor *
mife_opes(BLIN_flag flags, const char *src, size_t size) {
    mife_descriptor *mife = NULL;
    ifBLIN_FLAG = flags;
    size_t len;
    int i;

    ifBLIN_QX2("+ flags=%08X, size=%u\n", flags, (u_int32_t)size);
    SETBDEF(flags);
    if  (!(mife = malloc(sizeof(mife_descriptor)))) {
        errno = ENOMEM;
        ifBLIN_QW0("#1");
        goto out;
    }
    bzero(mife, sizeof(mife_descriptor));
    mife->flag = TRUFLAG(flags) | MIFE_INMM | MIFE_ALLC | MIFE_PIPE | MIFE_EOFL;
    mife->mindelta = 0;
    mife->fd = -1;
    mife->offset = 0;
    len = size;
    if  (!size && !(mife->flag & MIFE_STRI)) len = strlen(src);
    i = (flags & MIFE_FULL) ? 1 : 0;
    mife->actlen = len;
    if  (!(mife->buffer = malloc(mife->actlen + i))) {
        free(mife);
        mife = NULL;
        errno = ENOMEM;
        ifBLIN_QW0("#2");
        goto out;
    }
    bcopy(src, mife->buffer, mife->actlen);
    if  (i) ((char *)(mife->buffer))[mife->actlen] = 0;
    mife->buflen = mife->actlen + i;
    ifBLIN_QX2( "- flags=%08X, mindelta=%08X, buflen=%08X, actlen=%08\n"
              , mife->flag
              , (u_int32_t)(mife->mindelta)
              , (u_int32_t)(mife->buflen)
              , (u_int32_t)(mife->actlen)
              )
    ;
out:
    return(mife);
}

mife_descriptor *
mife_opem(BLIN_flag flags, char *src, size_t size) {
    mife_descriptor *mife = NULL;
    ifBLIN_FLAG = flags;
    size_t len;
    int i;

    ifBLIN_QX2("+ flags=%08X, size=%u\n", flags, (u_int32_t)size);
    SETBDEF(flags);
    if  (!(mife = malloc(sizeof(mife_descriptor)))) {
        errno = ENOMEM;
        ifBLIN_QW0("#1");
        goto out;
    }
    bzero(mife, sizeof(mife_descriptor));
    mife->flag = TRUFLAG(flags) | MIFE_INMM | MIFE_PIPE | MIFE_EOFL;
    mife->mindelta = 0;
    mife->fd = -1;
    mife->offset = 0;
    len = size;
    if  (!size && !(mife->flag & MIFE_STRI)) len = strlen(src);
    i = (mife->flag & MIFE_FULL) ? 1 : 0;
    mife->buffer = src;
    mife->actlen = len;
    if  (i) ((char *)(mife->buffer))[mife->actlen] = 0;
    mife->buflen = mife->actlen + i;
    ifBLIN_QX2( "- flags=%08X, mindelta=%08X, buflen=%08X, actlen=%08X\n"
              , mife->flag
              , (u_int32_t)(mife->mindelta)
              , (u_int32_t)(mife->buflen)
              , (u_int32_t)(mife->actlen)
              )
    ;
out:
    return(mife);
}
